# TPK4186 - Advanced Tools for Performance Engineering
# Binomials.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# 1. Recursive Algorithm
# ----------------------

def RecursiveBinomial(n, p):
  if p==0 or p==n:
    return 1
  return RecursiveBinomial(n-1, p-1) + RecursiveBinomial(n-1, p)

# 2. Dynamic Programming Algorithm
# --------------------------------

def DynamicProgrammingBinomial(n, p):
  results = [0 for i in range(0, p+1)]
  results[0] = 1
  for i in range(1, n+1):
    for j in range(p, 0, -1):
      results[j] = results[j] + results[j-1]
  return results[p]

# 3. Caching Algorithm
# --------------------

def CachingBinomial(n, p):
  cache = [(-1, -1, -1) for i in range(0, 1000)]
  return CachedBinomial(n, p, cache)

def CachedBinomial(n, p, cache):
  if p==0 or p==n:
    return 1
  hashCode = (13*n + 7*p) % len(cache)
  (nc, pc, rc) = cache[hashCode]
  if nc==n and pc==p:
    return rc
  r = CachedBinomial(n-1, p-1, cache) + CachedBinomial(n-1, p, cache)
  cache[hashCode] = (n, p, r)
  return r

# 4. Tests
# --------

print(RecursiveBinomial(6, 4))
print(RecursiveBinomial(30, 10)) # This calculation takes a while

print(DynamicProgrammingBinomial(6, 4))
print(DynamicProgrammingBinomial(30, 10))
print(DynamicProgrammingBinomial(100, 20))

print(CachingBinomial(6, 4))
print(CachingBinomial(30, 10))
print(CachingBinomial(100, 20))
