# TPK4186 - Advanced Tools for Performance Engineering
# IrisDataset.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of Contents
# -----------------
# 1. Required Modules
# 2. Loading of training and test sets
# 3. Statistics

# 2. Functions to print results
# 3. Loading of training and test sets
# 4. Logistic regression
# 5. Support Vector Machines
# 6. KNeighborsClassifier
# 7. Naive Bayes
# 8. Decision Trees
# 9. Multi-layer Perceptron


# 1. Required Modules
# -------------------

import numpy

from sklearn import datasets

# 2. Loading of training and test sets
# ------------------------------------

dataSet = datasets.load_iris()
sample = dataSet['data']
labels = dataSet['target']

# 3. Statistics
# -------------

sepalLengths = sample[:, 0]
min = sepalLengths.min()
max = sepalLengths.max()
mean = sepalLengths.mean()
print("Sepal lengths:\t{0:1.2f}\t{1:1.2f}\t{2:1.2f}".format(min, max, mean))

sepalWidths = sample[:, 1]
min = sepalWidths.min()
max = sepalWidths.max()
mean = sepalWidths.mean()
print("Sepal widths:\t{0:1.2f}\t{1:1.2f}\t{2:1.2f}".format(min, max, mean))

petalLengths = sample[:, 2]
min = petalLengths.min()
max = petalLengths.max()
mean = petalLengths.mean()
print("Petal lengths:\t{0:1.2f}\t{1:1.2f}\t{2:1.2f}".format(min, max, mean))

petalWidths = sample[:, 3]
min = petalWidths.min()
max = petalWidths.max()
mean = petalWidths.mean()
print("Petal widths:\t{0:1.2f}\t{1:1.2f}\t{2:1.2f}".format(min, max, mean))

# 4. Creation of training and test sets.
# --------------------------------------

state = numpy.random.get_state()
numpy.random.shuffle(sample)
numpy.random.set_state(state)
numpy.random.shuffle(labels)

trainingSample = sample[0:120, :]
trainingLabels = labels[0:120]

testSample = sample[120:150, :]
testLabels = labels[120:150]


# 5. Functions to print results
# -----------------------------

def PrintResults(actualStates, predictedLabels):
  successes = [0, 0, 0, 0]
  failures = [0, 0, 0, 0]
  actualCounts = [0, 0, 0]
  predictedCounts = [0, 0, 0]
  for i in range(0, len(actualStates)):
    if actualStates[i]==predictedLabels[i]:
      successes[int(actualStates[i])] += 1
      successes[3] += 1
    else:
      failures[int(actualStates[i])] += 1
      failures[3] += 1
    actualCounts[int(actualStates[i])]  += 1
    predictedCounts[int(predictedLabels[i])]  += 1
  print("Actual")
  for c in range(0, 3):
    print("Class {0:d}:\t{1:d}\t{2:.1f}".format(c, actualCounts[c], 100*actualCounts[c]/len(actualStates)))
  print("Predicted")
  for c in range(0, 3):
    print("Class {0:d}:\t{1:d}\t{2:.1f}".format(c, predictedCounts[c], 100*predictedCounts[c]/len(actualStates)))
  print("Successes")
  for c in range(0, 3):
    print("Class {0:d}:\t{1:d}\t{2:.1f}".format(c, successes[c], 100*successes[c]/actualCounts[c]))
  print("Total:\t{0:d}\t{1:.1f}".format(successes[3], 100*successes[3]/len(actualStates)))
  print("Failures")
  for c in range(0, 3):
    print("Class {0:d}:\t{1:d}\t{2:.1f}".format(c, failures[c], 100*failures[c]/actualCounts[c]))
  print("Total:\t{0:d}\t{1:.1f}".format(failures[3], 100*failures[3]/len(actualStates)))

# 4. Logistic Regression
# ----------------------

from sklearn import linear_model

print("Logistic Regression: Start")
model = linear_model.LogisticRegression()
model.fit(trainingSample, trainingLabels)
print("Logistic Regression: Training set learned")

predictedLabels = model.predict(testSample)
PrintResults(testLabels, predictedLabels)

# 5. Support Vector Machines
# --------------------------

from sklearn import svm

#print("Support Vector Machines: Start")
#model = svm.SVC()
#model.fit(trainingSample, trainingLabels)
#print("Training set learned")
#
#predictedLabels = model.predict(testSample)
#PrintResults(testLabels, predictedLabels)


# 6. KNeighborsClassifier
# -----------------------

from sklearn.neighbors import KNeighborsClassifier

#print("KNeighborsClassifier: Start")
#model = KNeighborsClassifier(n_neighbors=3)
#model.fit(trainingSample, trainingLabels)
#print("Training set learned")
#
#predictedLabels = model.predict(testSample)
#PrintResults(testLabels, predictedLabels)
#
# 7. Naive Bayes
# --------------

from sklearn.naive_bayes import GaussianNB

#print("Naive Bayes Classifier: Start")
#model = GaussianNB()
#model.fit(trainingSample, trainingLabels)
#print("Training set learned")
#
#predictedLabels = model.predict(testSample)
#PrintResults(testLabels, predictedLabels)

# 8. Decision Trees
# -----------------

#from sklearn.tree import DecisionTreeClassifier
#print("Decision Trees Classifier: Start")
#model = DecisionTreeClassifier()
#model.fit(trainingSample, trainingLabels)
#print("Training set learned")
#
#predictedLabels = model.predict(testSample)
#PrintResults(testLabels, predictedLabels)

# 9. Multi-layer Perceptron
# -------------------------

from sklearn.neural_network import MLPClassifier
print("Multi-layer Perceptron: Start")
model = MLPClassifier(max_iter=1000)
model.fit(trainingSample, trainingLabels)
print("Training set learned")

predictedLabels = model.predict(testSample)
PrintResults(testLabels, predictedLabels)

