# TPK4186 - Advanced Tools for Performance Engineering
# KOutOfN.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

import random

# 1. Recursive Algorithm
# ----------------------

def RecursiveProbability(n, k, Ps):
  if k>n:
    return 0
  if k==0:
    return 1
  return Ps[n]*RecursiveProbability(n-1, k-1, Ps) + (1-Ps[n])*RecursiveProbability(n-1, k, Ps)

# 2. Dynamic Programming Algorithm
# --------------------------------

def DynamicProgrammingProbability(n, k, Ps):
  results = [0 for i in range(0, k+1)]
  results[0] = 1.0
  for i in range(1, n+1):
    results[k] += Ps[i]*results[k-1]
    for j in range(k-1, 0, -1):
      results[j] = Ps[i]*results[j-1] + (1-Ps[i])*results[j]
    results[0] = (1-Ps[i])*results[0]
  return results[k]

# 3. Caching Algorithm
# --------------------

def CachingProbability(n, k, Ps):
  cache = [(-1, -1, -1) for i in range(0, 1000)]
  return CachedProbability(n, k, Ps, cache)

def CachedProbability(n, k, Ps, cache):
  if k>n:
    return 0
  if k==0:
    return 1
  hashCode = (13*n + 7*k) % len(cache)
  (nc, kc, rc) = cache[hashCode]
  if nc==n and kc==k:
    return rc
  r = Ps[n]*CachedProbability(n-1, k-1, Ps, cache) + (1-Ps[n])*CachedProbability(n-1, k, Ps, cache)
  cache[hashCode] = (n, k, r)
  return r

# 4. Tests
# --------

n = 5
k = 3
Ps = [0.9 for i in range(0, n+1)]
print(RecursiveProbability(n, k, Ps))
print(DynamicProgrammingProbability(n, k, Ps))
print(CachingProbability(n, k, Ps))

n = 5
k = 3
Ps = [random.random() for i in range(0, n+1)]

print(RecursiveProbability(n, k, Ps))
print(DynamicProgrammingProbability(n, k, Ps))
print(CachingProbability(n, k, Ps))
