# TPK4186 - Advanced Tools for Performance Engineering
# ProcessMonitoring.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of Contents
# -----------------
# 1. Required Modules
# 2. Functions to print results
# 3. Loading of training and test sets
# 4. Logistic regression
# 5. Support Vector Machines
# 6. KNeighborsClassifier
# 7. Naive Bayes
# 8. Decision Trees
# 9. Multi-layer Perceptron


# 1. Required Modules
# -------------------

import numpy

# 2. Functions to print results
# -----------------------------

def PrintStatistics(labels):
  counts = [0, 0]
  for i in range(0, len(labels)):
    counts[int(labels[i])] += 1
  for c in range(0, 2):
    print("Label {0:d}:\t{1:d}\t{2:.1f}".format(c, counts[c], 100*counts[c]/len(labels)))

def PrintResults(actualLabels, predictedLabels):
  successes = [0, 0, 0]
  failures = [0, 0, 0]
  actualCounts = [0, 0]
  predictedCounts = [0, 0]
  for i in range(0, len(actualLabels)):
    if actualLabels[i]==predictedLabels[i]:
      successes[int(actualLabels[i])] += 1
      successes[2] += 1
    else:
      failures[int(actualLabels[i])] += 1
      failures[2] += 1
    actualCounts[int(actualLabels[i])]  += 1
    predictedCounts[int(predictedLabels[i])]  += 1
  print("Actual")
  print("Label 0:\t{0:d}\t{1:.1f}".format(actualCounts[0], 100*actualCounts[0]/len(actualLabels)))
  print("Label 1:\t{0:d}\t{1:.1f}".format(actualCounts[1], 100*actualCounts[1]/len(actualLabels)))
  print("Predicted")
  print("Label 0:\t{0:d}\t{1:.1f}".format(predictedCounts[0], 100*predictedCounts[0]/len(actualLabels)))
  print("Label 1:\t{0:d}\t{1:.1f}".format(predictedCounts[1], 100*predictedCounts[1]/len(actualLabels)))
  print("Successes")
  print("Label 0:\t{0:d}\t{1:.1f}".format(successes[0], 100*successes[0]/actualCounts[0]))
  print("Label 1:\t{0:d}\t{1:.1f}".format(successes[1], 100*successes[1]/actualCounts[1]))
  print("Total:\t{0:d}\t{1:.1f}".format(successes[2], 100*successes[2]/len(actualLabels)))
  print("Failures")
  print("Label 0:\t{0:d}\t{1:.1f}".format(failures[0], 100*failures[0]/actualCounts[0]))
  print("Label 1:\t{0:d}\t{1:.1f}".format(failures[1], 100*failures[1]/actualCounts[1]))
  print("Total:\t{0:d}\t{1:.1f}".format(failures[2], 100*failures[2]/len(actualLabels)))

# 3. Loading of training and test sets
# ------------------------------------

VERSION = 7
trainingSet = numpy.genfromtxt("trainingSet{0:d}.csv".format(VERSION))
trainingSample = trainingSet[:, :-1]
trainingLabels = trainingSet[:, -1]

testSet = numpy.genfromtxt("testSet{0:d}.csv".format(VERSION))
testSample = testSet[:, :-1]
testLabels = testSet[:, -1]

PrintStatistics(trainingLabels)
PrintStatistics(testLabels)

# 4. Logistic Regression
# ----------------------

from sklearn import linear_model

print("Logistic Regression: Start")
model = linear_model.LogisticRegression()
model.fit(trainingSample, trainingLabels)
print("Logistic Regression: Training set learned")

predictedLabels = model.predict(testSample)
PrintResults(testLabels, predictedLabels)

# 5. Support Vector Machines
# --------------------------

from sklearn import svm

print("Support Vector Machines: Start")
model = svm.SVC()
model.fit(trainingSample, trainingLabels)
print("Training set learned")

predictedLabels = model.predict(testSample)
PrintResults(testLabels, predictedLabels)


# 6. KNeighborsClassifier
# -----------------------

from sklearn.neighbors import KNeighborsClassifier

print("KNeighborsClassifier: Start")
model = KNeighborsClassifier(n_neighbors=3)
model.fit(trainingSample, trainingLabels)
print("Training set learned")

predictedLabels = model.predict(testSample)
PrintResults(testLabels, predictedLabels)

# 7. Naive Bayes
# --------------

from sklearn.naive_bayes import GaussianNB

print("Naive Bayes Classifier: Start")
model = GaussianNB()
model.fit(trainingSample, trainingLabels)
print("Training set learned")

predictedLabels = model.predict(testSample)
PrintResults(testLabels, predictedLabels)

# 8. Decision Trees
# -----------------

from sklearn.tree import DecisionTreeClassifier
print("Decision Trees Classifier: Start")
model = DecisionTreeClassifier()
model.fit(trainingSample, trainingLabels)
print("Training set learned")

predictedLabels = model.predict(testSample)
PrintResults(testLabels, predictedLabels)

# 9. Multi-layer Perceptron
# -------------------------

from sklearn.neural_network import MLPClassifier
print("Multi-layer Perceptron: Start")
model = MLPClassifier(max_iter=1000)
model.fit(trainingSample, trainingLabels)
print("Training set learned")

predictedLabels = model.predict(testSample)
PrintResults(testLabels, predictedLabels)

